"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
describe('Option', function () {
    function getAssertion(type) {
        return function (scenario) {
            it('correctly creates an instance of Some with value \'' + scenario.value + '\'', function () {
                var subject = _1.Some(scenario.value);
                expect(subject instanceof _1._Some).toEqual(true);
                expect(subject.is_some()).toEqual(true);
                expect(subject.is_none()).toEqual(false);
                expect(subject.unwrap_or({})).toEqual(scenario.value);
                if (_1.is_some(subject)) {
                    expect(typeof (subject.unwrap())).toEqual(type.toLowerCase());
                    expect(subject.unwrap()).toEqual(scenario.value);
                }
                else {
                    throw new Error('Has to be _Some!');
                }
            });
        };
    }
    describe('Some', function () {
        describe('Boolean', function () {
            var type = 'Boolean';
            var scenarios = [
                { value: true },
                { value: false },
                { value: Boolean(true) }
            ];
            var assertion = getAssertion(type);
            scenarios.forEach(assertion);
        });
        describe('Number', function () {
            var type = 'Number';
            var scenarios = [
                { value: 37 },
                { value: 3.14 },
                { value: 0 },
                { value: Math.LN2 },
                { value: Infinity },
                { value: NaN },
                { value: Number(1) },
            ];
            var assertion = getAssertion(type);
            scenarios.forEach(assertion);
        });
        describe('String', function () {
            var type = 'String';
            var scenarios = [
                { value: '' },
                { value: 'bla' },
                { value: typeof 1 },
                { value: String('abc') },
            ];
            var assertion = getAssertion(type);
            scenarios.forEach(assertion);
        });
        describe('Function', function () {
            var type = 'Function';
            var scenarios = [
                {
                    value: function () {
                    }
                },
                {
                    value: (function () {
                        function C() {
                        }
                        return C;
                    }())
                },
                { value: Math.sin },
            ];
            var assertion = getAssertion(type);
            scenarios.forEach(assertion);
        });
        describe('Object', function () {
            var type = 'Object';
            var scenarios = [
                { value: { a: 1 } },
                { value: [1, 2, 4] },
                { value: new Date() },
                { value: new Boolean(true) },
                { value: new Number(1) },
                { value: new String('abc') },
            ];
            var assertion = getAssertion(type);
            scenarios.forEach(assertion);
        });
        describe('RegEx', function () {
            var val = /s/;
            it('correctly creates an instance of Option with value \'' + val + '\'', function () {
                var subject = _1.Some(val);
                expect(subject instanceof _1._Some).toEqual(true);
                expect(subject.is_some()).toEqual(true);
                expect(subject.is_none()).toEqual(false);
                expect(subject.unwrap_or(val)).toEqual(val);
                if (_1.is_some(subject)) {
                    var type = typeof (subject.unwrap());
                    expect(type === 'function' || type === 'object').toEqual(true);
                    expect(subject.unwrap()).toEqual(val);
                }
                else {
                    throw new Error('Has to be _Some!');
                }
            });
        });
        describe('Undefined, Null', function () {
            var array = ['a', 'b'];
            var outOfBoundIndex = array.length + 1;
            var object = {
                a: '_a',
                b: '_b'
            };
            var outOfBoundProperty = 'z';
            var scenarios = [
                { value: undefined },
                { value: null },
                { value: array[outOfBoundIndex] },
                { value: [null][0] },
                { value: object[outOfBoundProperty] },
                { value: { _: null }._ },
            ];
            var assertion = function (scenario) {
                it('is None when trying to access out of bound index, property or variable, calling unwrap() impossible', function () {
                    var subject = _1.Some(scenario.value);
                    expect(subject instanceof _1._None).toEqual(true);
                    expect(subject.is_none()).toEqual(true);
                    expect(subject.is_some()).toEqual(false);
                    expect(function () { return subject.unwrap(); }).toThrow();
                });
            };
            scenarios.forEach(assertion);
        });
        describe('match', function () {
            it('correctly matches Some and returns transformed value', function () {
                var string = _1.Some('string');
                var subject = string.match({
                    some: function (_) { return _.toUpperCase(); },
                    none: 'OTHER STRING'
                });
                expect(subject).toEqual('STRING');
            });
            it('correctly matches None and returns fallback value', function () {
                var arr = [1, 2, 3];
                var maybeNumber = _1.Some(arr[arr.length + 1]);
                var subject = maybeNumber.match({
                    some: function (_) { return _ * 2; },
                    none: NaN
                });
                expect(subject).toEqual(NaN);
            });
        });
        describe('map', function () {
            it('correctly maps Some and returns a new Some with transformed value', function () {
                var string = _1.Some('123');
                var subject = string.map(function (_) { return parseInt(_); });
                expect(_1.is_some(subject) ? subject.unwrap() : undefined).toEqual(123);
            });
        });
    });
    describe('None', function () {
        it('correctly creates its instance, returns correct value when calling unwrap_or()', function () {
            var subject = _1.None;
            expect(subject instanceof _1._None).toEqual(true);
            expect(subject.is_none()).toEqual(true);
            expect(subject.is_some()).toEqual(false);
            expect(subject.unwrap_or('string')).toEqual('string');
        });
        describe('unwrap_or', function () {
            it('should correctly throw if trying to call with undefined or null', function () {
                var subject = _1.None;
                var array = ['a'], outOfBoundIndex = array.length + 1;
                expect(function () { return subject.unwrap_or(array[outOfBoundIndex]); }).toThrow();
            });
        });
        describe('match', function () {
            it('correctly matches None and returns fallback value', function () {
                var subject = _1.None.match({
                    some: function (_) { return 'something'; },
                    none: 'nothing'
                });
                expect(subject).toEqual('nothing');
            });
        });
        describe('map', function () {
            it('correctly maps Some and returns a new Some with transformed value', function () {
                var subject = _1.None.map(function (_) { return parseInt(_); });
                expect(subject instanceof _1._None).toEqual(true);
            });
        });
    });
    describe('Option', function () {
        describe('match', function () {
            it('correctly matches Some and returns transformed value', function () {
                var a;
                var date = new Date();
                if (true === true)
                    a = _1.Some(date);
                else
                    a = _1.None;
                var subject = a.match({
                    some: function (_) { return _.getFullYear(); },
                    none: 1994
                });
                expect(subject).toEqual(date.getFullYear());
            });
            it('correctly matches None and returns fallback value', function () {
                var a;
                var initialValue = true;
                if (1 > 2)
                    a = _1.Some(initialValue);
                else
                    a = _1.None;
                var subject = a.match({
                    some: function (_) { return !_; },
                    none: initialValue
                });
                expect(subject).toEqual(initialValue);
            });
            it('correctly matches None and returns fallback value when method provided to none branch', function () {
                var a = _1.None;
                var subject = a.match({
                    some: function (_) { return _; },
                    none: function () { return 'N/A'; }
                });
                expect(subject).toEqual('N/A');
            });
        });
        describe('map', function () {
            it('returns transformed Some when method applied to a value that exists', function () {
                var arr = [1, 2, 3];
                var subject = _1.Some(arr[0])
                    .map(function (_) { return _.toString(); });
                expect(_1.is_some(subject) ? subject.unwrap() : undefined).toEqual('1');
            });
            it('returns None when method applied to a value that does not exist', function () {
                var arr = [1, 2, 3];
                var subject = _1.Some(arr[arr.length + 1])
                    .map(function (_) { return _.toString(); });
                expect(subject.is_none()).toEqual(true);
            });
            it('returns None when method throws', function () {
                var arr = [1, 2, 3];
                var subject = _1.Some(arr[0])
                    .map(function (_) { return JSON.parse('{null}'); });
                expect(subject.is_none()).toEqual(true);
            });
        });
    });
    describe('unwrap', function () {
        it('throws if no value inside', function () {
            var string = new _1._Some(null);
            expect(function () { return string.unwrap(); }).toThrow();
        });
    });
    describe('unwrap_or', function () {
        it('throws if no value provided', function () {
            var string = new _1._Some(null);
            expect(function () { return string.unwrap_or(null); }).toThrow();
        });
    });
});
describe('is_some', function () {
    it('should unwrap after a successful preliminary check', function () {
        var a;
        var subject = _1.Some(42);
        if (_1.is_some(subject)) {
            a = subject.unwrap();
        }
        else {
            a = 0;
        }
        expect(a).toEqual(42);
    });
    it('should not unwrap after a failing preliminary check', function () {
        var a = ['a', 'b', 'c'], outOfBoundIndex = a.length + 1;
        var b;
        var subject = _1.Some(a[outOfBoundIndex]);
        if (_1.is_some(subject)) {
            b = subject.unwrap();
        }
        else {
            b = 'This Is None';
        }
        expect(b).toEqual('This Is None');
    });
});
describe('is_none', function () {
    it('should return true if Option is None', function () {
        var a = ['a', 'b', 'c'], outOfBoundIndex = a.length + 1;
        var b;
        var subject = _1.Some(a[outOfBoundIndex]);
        if (_1.is_none(subject)) {
            b = 'Correct';
        }
        else {
            b = 'Fail';
        }
        expect(_1.is_none(subject)).toEqual(true);
        expect(b).toEqual('Correct');
    });
    it('should return false if Option is Some', function () {
        var a = function () { return null; };
        var b;
        var subject = _1.Some(a);
        if (_1.is_none(subject)) {
            b = 'Fail';
        }
        else {
            b = 'Correct';
        }
        expect(_1.is_none(subject)).toEqual(false);
        expect(b).toEqual('Correct');
    });
    it('should return true if value is None', function () {
        var a;
        var subject = _1.None;
        if (_1.is_none(subject)) {
            a = 'Correct';
        }
        else {
            a = 'Fail';
        }
        expect(_1.is_none(subject)).toEqual(true);
        expect(a).toEqual('Correct');
    });
});
describe('get_in', function () {
    it('correctly returns a Some if key found in nested object', function () {
        var obj = { a: { b: 'val' } };
        var subject = _1.get_in(obj, 'a.b');
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or('')).toEqual('val');
    });
    it('correctly returns a None if key not found in nested object', function () {
        var obj = { a: { b: 'val' } };
        var subject = _1.get_in(obj, 'a.nonExistentKey');
        expect(subject.is_none()).toEqual(true);
    });
    it('correctly returns a None if object itself undefined', function () {
        var subject = _1.get_in(undefined, 'a.b.c.d');
        expect(subject.is_none()).toEqual(true);
    });
});
describe('and_then', function () {
    var contact;
    var driver;
    var truck;
    beforeEach(function () {
        contact = { name: _1.None };
        driver = { contact: _1.None };
        truck = { driver: _1.None };
    });
    it('correctly returns the name as a Some if all properties are Some', function () {
        contact.name = _1.Some('Name');
        driver.contact = _1.Some(contact);
        truck.driver = _1.Some(driver);
        var subject = truck.driver
            .and_then(function (_) { return _.contact; })
            .and_then(function (_) { return _.name; });
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or('')).toEqual('Name');
    });
    it('correctly returns None if \'contact.name\' is None', function () {
        driver.contact = _1.Some(contact);
        truck.driver = _1.Some(driver);
        var subject = truck.driver
            .and_then(function (_) { return _.contact; })
            .and_then(function (_) { return _.name; });
        expect(subject.is_none()).toEqual(true);
    });
    it('correctly returns None if \'contact\' is None', function () {
        contact.name = _1.Some('Name');
        driver.contact = _1.None;
        truck.driver = _1.Some(driver);
        var subject = truck.driver
            .and_then(function (_) { return _.contact; })
            .and_then(function (_) { return _.name; });
        expect(subject.is_none()).toEqual(true);
    });
    it('correctly returns None if \'driver\' is None', function () {
        contact.name = _1.Some('Name');
        driver.contact = _1.Some(contact);
        truck.driver = _1.None;
        var subject = truck.driver
            .and_then(function (_) { return _.contact; })
            .and_then(function (_) { return _.name; });
        expect(subject.is_none()).toEqual(true);
    });
    it('catches error and returns None if method throws', function () {
        contact.name = _1.Some('Name');
        driver.contact = _1.Some(contact);
        truck.driver = _1.Some(driver);
        var subject = truck.driver
            .and_then(function (_) { return _1.Some(JSON.parse('{null}')); });
        expect(subject.is_none()).toEqual(true);
    });
});
describe('or', function () {
    it('correctly returns Some(a) if \'a\' is Some and \'b\' is None', function () {
        var a = _1.Some(123), b = _1.None;
        var subject = a.or(b);
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or(0)).toEqual(123);
    });
    it('correctly returns Some(b) if \'a\' is None and \'b\' is Some', function () {
        var a = _1.None, b = _1.Some(456);
        var subject = a.or(b);
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or(0)).toEqual(456);
    });
    it('correctly returns Some(a) if \'a\' is Some and \'b\' is Some', function () {
        var a = _1.Some(11), b = _1.Some(12);
        var subject = a.or(b);
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or(0)).toEqual(11);
    });
    it('correctly returns None if \'a\' is None and \'b\' is None', function () {
        var a = _1.None, b = _1.None;
        var subject = a.or(b);
        expect(subject.is_none()).toEqual(true);
    });
});
describe('and', function () {
    it('correctly returns None if \'a\' is Some and \'b\' is None', function () {
        var a = _1.Some(123), b = _1.None;
        var subject = a.and(b);
        expect(subject.is_none()).toEqual(true);
    });
    it('correctly returns None if \'a\' is None and \'b\' is Some', function () {
        var a = _1.None, b = _1.Some(123);
        var subject = a.and(b);
        expect(subject.is_none()).toEqual(true);
    });
    it('correctly returns Some(b) if \'a\' is Some and \'b\' is Some', function () {
        var a = _1.Some(123), b = _1.Some(456);
        var subject = a.and(b);
        expect(subject.is_some()).toEqual(true);
        expect(subject.unwrap_or(0)).toEqual(456);
    });
    it('correctly returns None if \'a\' is None and \'b\' is None', function () {
        var a = _1.None, b = _1.None;
        var subject = a.and(b);
        expect(subject.is_none()).toEqual(true);
    });
});
//# sourceMappingURL=index.spec.js.map