"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _1 = require(".");
describe('Result', function () {
    function getOkAssertion(type) {
        return function (scenario) {
            it('correctly creates an instance of Ok with value \'' + scenario.value + '\'', function () {
                var subject = _1.Ok(scenario.value);
                expect(subject instanceof _1._Ok).toEqual(true);
                expect(subject.is_ok()).toEqual(true);
                expect(subject.is_err()).toEqual(false);
                expect(function () { return subject.unwrap_err(); }).toThrow();
                expect(subject.unwrap_or('')).toEqual(scenario.value);
                if (_1.is_ok(subject)) {
                    expect(typeof (subject.unwrap())).toEqual(type.toLowerCase());
                    expect(subject.unwrap()).toEqual(scenario.value);
                }
                else {
                    throw new Error('Has to be _Ok!');
                }
            });
        };
    }
    function getErrAssertion(type) {
        return function (scenario) {
            it('correctly creates an instance of Err with value \'' + scenario.value + '\'', function () {
                var subject = _1.Err(scenario.value);
                expect(subject instanceof _1._Err).toEqual(true);
                expect(subject.is_ok()).toEqual(false);
                expect(subject.is_err()).toEqual(true);
                expect(function () { return subject.unwrap(); }).toThrow();
                expect(subject.unwrap_or('optb')).toEqual('optb');
                if (_1.is_err(subject)) {
                    expect(typeof (subject.unwrap_err())).toEqual(type.toLowerCase());
                    expect(subject.unwrap_err()).toEqual(scenario.value);
                }
                else {
                    throw new Error('Has to be _Err!');
                }
            });
        };
    }
    describe('Boolean', function () {
        var type = 'Boolean';
        var scenarios = [
            { value: true },
            { value: false },
            { value: Boolean(true) }
        ];
        var assertionOk = getOkAssertion(type), assertionErr = getErrAssertion(type);
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('Number', function () {
        var type = 'Number';
        var scenarios = [
            { value: 37 },
            { value: 3.14 },
            { value: 0 },
            { value: Math.LN2 },
            { value: Infinity },
            { value: NaN },
            { value: Number(1) },
        ];
        var assertionOk = getOkAssertion(type), assertionErr = getErrAssertion(type);
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('String', function () {
        var type = 'String';
        var scenarios = [
            { value: '' },
            { value: 'bla' },
            { value: typeof 1 },
            { value: String('abc') },
        ];
        var assertionOk = getOkAssertion(type), assertionErr = getErrAssertion(type);
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('Function', function () {
        var type = 'Function';
        var scenarios = [
            {
                value: function () {
                }
            },
            {
                value: (function () {
                    function C() {
                    }
                    return C;
                }())
            },
            { value: Math.sin },
        ];
        var assertionOk = getOkAssertion(type), assertionErr = getErrAssertion(type);
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('Object', function () {
        var type = 'Object';
        var scenarios = [
            { value: { a: 1 } },
            { value: [1, 2, 4] },
            { value: new Date() },
            { value: new Boolean(true) },
            { value: new Number(1) },
            { value: new String('abc') },
        ];
        var assertionOk = getOkAssertion(type), assertionErr = getErrAssertion(type);
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('RegEx', function () {
        var val = /s/;
        it('correctly creates an instance of Ok with value \'' + val + '\'', function () {
            var subject = _1.Ok(val);
            expect(subject instanceof _1._Ok).toEqual(true);
            expect(subject.is_ok()).toEqual(true);
            expect(subject.is_err()).toEqual(false);
            if (_1.is_ok(subject)) {
                var type = typeof (subject.unwrap());
                expect(type === 'function' || type === 'object').toEqual(true);
                expect(subject.unwrap()).toEqual(val);
            }
            else {
                throw new Error('Has to be _Ok!');
            }
        });
        it('correctly creates an instance of Err with value \'' + val + '\'', function () {
            var subject = _1.Err(val);
            expect(subject instanceof _1._Err).toEqual(true);
            expect(subject.is_ok()).toEqual(false);
            expect(subject.is_err()).toEqual(true);
            if (_1.is_err(subject)) {
                var type = typeof (subject.unwrap_err());
                expect(type === 'function' || type === 'object').toEqual(true);
                expect(subject.unwrap_err()).toEqual(val);
            }
            else {
                throw new Error('Has to be _Err!');
            }
        });
    });
    describe('Undefined, Null', function () {
        var array = ['a', 'b'], outOfBoundIndex = array.length + 1;
        var object = { a: '_a', b: '_b' }, outOfBoundProperty = 'z';
        var scenarios = [
            { value: undefined },
            { value: null },
            { value: array[outOfBoundIndex] },
            { value: [null][0] },
            { value: object[outOfBoundProperty] },
            { value: { _: null }._ },
        ];
        var assertionOk = function (scenario) {
            it('Ok works correctly', function () {
                var subject = _1.Ok(scenario.value);
                expect(subject instanceof _1._Ok).toEqual(true);
                expect(subject.is_ok()).toEqual(true);
                expect(subject.is_err()).toEqual(false);
                expect(subject.unwrap()).toEqual(scenario.value);
            });
        };
        var assertionErr = function (scenario) {
            it('Err works correctly', function () {
                var subject = _1.Err(scenario.value);
                expect(subject instanceof _1._Err).toEqual(true);
                expect(subject.is_ok()).toEqual(false);
                expect(subject.is_err()).toEqual(true);
                expect(subject.unwrap_err()).toEqual(scenario.value);
            });
        };
        scenarios.forEach(assertionOk);
        scenarios.forEach(assertionErr);
    });
    describe('unwrap_or', function () {
        it('throws if no value provided', function () {
            var string = _1.Ok(null);
            expect(function () { return string.unwrap_or(null); }).toThrow();
            string = _1.Err('string');
            expect(function () { return string.unwrap_or(null); }).toThrow();
        });
    });
    describe('match', function () {
        it('correctly matches Ok and returns transformed value', function () {
            var string = _1.Ok('string');
            var subject = string.match({
                ok: function (_) { return _.toUpperCase(); },
                err: function (_) { return _; }
            });
            expect(subject).toEqual('STRING');
        });
        it('correctly matches Err and returns fallback value', function () {
            var arr = [1, 2, 3];
            var number = _1.Err(arr[0]);
            var subject = number.match({
                ok: function (_) { return 0; },
                err: function (_) { return _; }
            });
            expect(subject).toEqual(1);
        });
        it('correctly matches Result and returns fallback value', function () {
            function getMessage(data) {
                return data.match({
                    ok: function (_) { return "Success: " + _; },
                    err: function (_) { return "Error: " + _; }
                });
            }
            expect(getMessage(_1.Ok('ok'))).toEqual("Success: ok");
            expect(getMessage(_1.Err('err'))).toEqual("Error: err");
        });
    });
    describe('map', function () {
        it('correctly maps Ok and returns transformed Result', function () {
            var string = _1.Ok('123');
            var subject = string.map(function (_) { return parseInt(_); });
            expect(subject.unwrap()).toEqual(123);
        });
        it('correctly returns untouched Err when trying to use map', function () {
            var arr = [1, 2, 3];
            var number = _1.Err(arr[0]);
            var subject = number.map(function (_) { return _.toString(); });
            expect(subject.unwrap_err()).toEqual(1);
        });
        it('correctly maps Result and returns transformed value', function () {
            function getMessage(data) {
                return data.map(function (_) { return parseInt(_); });
            }
            var subject = getMessage(_1.Ok('123'));
            expect(_1.is_ok(subject) ? subject.unwrap() : 0).toEqual(123);
            subject = getMessage(_1.Err('123'));
            expect(_1.is_err(subject) ? subject.unwrap_err() : '0').toEqual('123');
        });
    });
    describe('ok', function () {
        it('correctly returns Some when Result is ok', function () {
            var string_ok = _1.Ok('123');
            var subject = string_ok.ok();
            expect(subject.is_some()).toEqual(true);
            expect(subject.unwrap()).toEqual('123');
        });
        it('correctly returns None when Result is err', function () {
            var string_err = _1.Err('123');
            var subject = string_err.ok();
            expect(subject.is_none()).toEqual(true);
        });
    });
    describe('err', function () {
        it('correctly returns Some when Result is err', function () {
            var string_err = _1.Err('123');
            var subject = string_err.err();
            expect(subject.is_some()).toEqual(true);
            expect(subject.unwrap()).toEqual('123');
        });
        it('correctly returns None when Result is ok', function () {
            var string_ok = _1.Ok('123');
            var subject = string_ok.err();
            expect(subject.is_none()).toEqual(true);
        });
    });
});
//# sourceMappingURL=index.spec.js.map